@@ -1648,6 +1648,59 @@ scrape_configs:
 			ProxyAuthConfig: &promauth.Config{},
 		},
 	})
+	f(`
+global:
+  scrape_timeout: 1d
+scrape_configs:
+- job_name: foo
+  scrape_interval: 1w
+  scrape_align_interval: 1d
+  scrape_offset: 2d
+  static_configs:
+  - targets: ["foo.bar:1234"]
+`, []*ScrapeWork{
+		{
+			ScrapeURL:           "http://foo.bar:1234/metrics",
+			ScrapeInterval:      time.Hour * 24 * 7,
+			ScrapeTimeout:       time.Hour * 24,
+			ScrapeAlignInterval: time.Hour * 24,
+			ScrapeOffset:        time.Hour * 24 * 2,
+			HonorTimestamps:     true,
+			Labels: []prompbmarshal.Label{
+				{
+					Name:  "__address__",
+					Value: "foo.bar:1234",
+				},
+				{
+					Name:  "__metrics_path__",
+					Value: "/metrics",
+				},
+				{
+					Name:  "__scheme__",
+					Value: "http",
+				},
+				{
+					Name:  "__scrape_interval__",
+					Value: "168h0m0s",
+				},
+				{
+					Name:  "__scrape_timeout__",
+					Value: "24h0m0s",
+				},
+				{
+					Name:  "instance",
+					Value: "foo.bar:1234",
+				},
+				{
+					Name:  "job",
+					Value: "foo",
+				},
+			},
+			AuthConfig:      &promauth.Config{},
+			ProxyAuthConfig: &promauth.Config{},
+			jobNameOriginal: "foo",
+		},
+	})
 }
 
 func equalStaticConfigForScrapeWorks(a, b []*ScrapeWork) bool {
