@@ -370,8 +370,7 @@ func (mn *MetricName) Marshal(dst []byte) []byte {
 		tag := &mn.Tags[i]
 		requiredSize += len(tag.Key) + len(tag.Value) + 2
 	}
-	dst = bytesutil.Resize(dst, requiredSize)
-	dst = dst[:dstLen]
+	dst = bytesutil.ResizeWithCopyMayOverallocate(dst, requiredSize)[:dstLen]
 
 	// Marshal MetricGroup
 	dst = marshalTagValue(dst, mn.MetricGroup)
@@ -478,7 +477,7 @@ func MarshalMetricNameRaw(dst []byte, labels []prompb.Label) []byte {
 		dstSize += len(label.Value)
 		dstSize += 4
 	}
-	dst = bytesutil.Resize(dst, dstSize)[:dstLen]
+	dst = bytesutil.ResizeWithCopyMayOverallocate(dst, dstSize)[:dstLen]
 
 	// Marshal labels to dst.
 	for i := range labels {
@@ -511,6 +510,8 @@ func trackDroppedLabels(labels, droppedLabels []prompb.Label) {
 	atomic.AddUint64(&MetricsWithDroppedLabels, 1)
 	select {
 	case <-droppedLabelsLogTicker.C:
+		// Do not call logger.WithThrottler() here, since this will result in increased CPU usage
+		// because labelsToString() will be called with each trackDroppedLAbels call.
 		logger.Warnf("dropping %d labels for %s; dropped labels: %s; either reduce the number of labels for this metric "+
 			"or increase -maxLabelsPerTimeseries=%d command-line flag value",
 			len(droppedLabels), labelsToString(labels), labelsToString(droppedLabels), maxLabelsPerTimeseries)
@@ -607,6 +608,12 @@ func unmarshalBytesFast(src []byte) ([]byte, []byte, error) {
 
 // sortTags sorts tags in mn to canonical form needed for storing in the index.
 //
+// The sortTags tries moving job-like tag to mn.Tags[0], while instance-like tag to mn.Tags[1].
+// See commonTagKeys list for job-like and instance-like tags.
+// This guarantees that indexdb entries for the same (job, instance) are located
+// close to each other on disk. This reduces disk seeks and disk read IO when metrics
+// for a particular job and/or instance are read from the disk.
+//
 // The function also de-duplicates tags with identical keys in mn. The last tag value
 // for duplicate tags wins.
 //
