@@ -49,7 +49,10 @@ func convertToCompositeTagFilters(tfs *TagFilters) []*TagFilters {
 			hasPositiveFilter = true
 		}
 	}
-	if len(names) == 0 {
+	// If tfs have no filters on __name__ or have no non-negative filters,
+	// then it is impossible to construct composite tag filter.
+	// See https://github.com/VictoriaMetrics/VictoriaMetrics/issues/2238
+	if len(names) == 0 || !hasPositiveFilter {
 		atomic.AddUint64(&compositeFilterMissingConversions, 1)
 		return []*TagFilters{tfs}
 	}
@@ -61,7 +64,7 @@ func convertToCompositeTagFilters(tfs *TagFilters) []*TagFilters {
 		tfsNew := make([]tagFilter, 0, len(tfs.tfs))
 		for _, tf := range tfs.tfs {
 			if len(tf.key) == 0 {
-				if !hasPositiveFilter || tf.isNegative {
+				if tf.isNegative {
 					// Negative filters on metric name cannot be used for building composite filter, so leave them as is.
 					tfsNew = append(tfsNew, tf)
 					continue
@@ -162,7 +165,7 @@ func (tfs *TagFilters) Add(key, value []byte, isNegative, isRegexp bool) error {
 	}
 	if isRegexp && string(value) == ".*" {
 		if !isNegative {
-			// Skip tag filter matching anything, since it equal to no filter.
+			// Skip tag filter matching anything, since it equals to no filter.
 			return nil
 		}
 
@@ -236,7 +239,7 @@ type tagFilter struct {
 	// contains the prefix for regexp filter if isRegexp==true.
 	regexpPrefix string
 
-	// Prefix always contains {nsPrefixTagToMetricIDs, key}.
+	// Prefix contains either {nsPrefixTagToMetricIDs, key} or {nsPrefixDateTagToMetricIDs, date, key}.
 	// Additionally it contains:
 	//  - value if !isRegexp.
 	//  - regexpPrefix if isRegexp.
@@ -375,6 +378,8 @@ func getCommonPrefix(ss []string) (string, []string) {
 
 // Init initializes the tag filter for the given commonPrefix, key and value.
 //
+// commonPrefix must contain either {nsPrefixTagToMetricIDs} or {nsPrefixDateTagToMetricIDs, date}.
+//
 // If isNegaitve is true, then the tag filter matches all the values
 // except the given one.
 //
